var $USER_FUNCTION$ = (function() {
  function formatObject(data, use_parse) {
    if (data && typeof data === 'object') {
      return data;
    } else {
      var ret;
      if (use_parse) {
        try {
          ret = JSON.parse(data);
        } catch (e) {
        }
      }
      if (!ret || typeof ret !== 'object') ret = {};
      return ret;
    }
  }

  function formatArray(data, use_parse) {
    if (Array.isArray(data)) {
      return data;
    } else {
      var ret;
      if (use_parse) {
        try {
          ret = JSON.parse(data);
        } catch (e) {
        }
      }
      if (!Array.isArray(ret)) ret = [];
      return ret;
    }
  }

  function formatDate(datetime) {
    var timestamp = +datetime;
    if (!timestamp) {
      return '';
    }
    var date = new Date(timestamp);
    return date.getFullYear() + '-' + ('0' + (date.getMonth() + 1)).slice(-2) +
      '-' + ('0' + date.getDate()).slice(-2);
  }

  return {
    docFuncs: [
      {
        name: 'taxi_days',
        meaning: '计算打车天数', // redmine #4729
        func: function(claim, input_path) {
          var taxi_type_ids = [497812, 486498];
          var rent_type_ids = [486496];
          var claim_line = formatArray(formatObject(claim).claim_line);
          var start_date = Date.parse(formatDate(claim.start_datetime));
          var end_date = Date.parse(formatDate(claim.end_datetime));
          var valid_dates = [];
          claim_line.forEach(function(line) {
            if (line && typeof line === 'object') {
              if (taxi_type_ids.indexOf(+line.type_id) >= 0) {
                var receipts = formatArray(line.receipts, true);
                receipts.forEach(function(rp) {
                  if (rp && typeof rp === 'object') {
                    var date = Date.parse(formatDate(rp.receipt_date || line.receipt_date));
                    if (date && typeof date === 'number'
                        && valid_dates.indexOf(date) < 0
                        && !(date <= start_date)
                        && !(date >= end_date)) valid_dates.push(date);
                  }
                });
              } else if (rent_type_ids.indexOf(+line.type_id) >= 0) {
                var line_start = Date.parse(formatDate(line.start_datetime));
                var line_end = Date.parse(formatDate(line.end_datetime));
                if (!line_start || !line_end) return;
                var day_ms = 24*3600*1000;
                for (var date = line_start; date <= line_end; date += day_ms) {
                  if (valid_dates.indexOf(date) < 0
                      && !(date <= start_date)
                      && !(date >= end_date)) valid_dates.push(date);
                }
              }
            }
          });
          var value = valid_dates.length;
          return [{
            path: input_path,
            value: String(value),
            protect: 'N'
          }];
        }
      }, {
        name: 'excessive_amount',
        meaning: '计算所有酒店费用的累计超标金额', // redmine #4733
        func: function(claim, input_path) {
          var hotel_type_ids = [476867];
          var claim_line = formatArray(formatObject(claim).claim_line);
          var amount = 0;
          claim_line.forEach(function(line) {
            if (line && typeof line === 'object'
                && hotel_type_ids.indexOf(+line.type_id) >= 0
                && typeof line.original_amount === 'number'
                && line.original_amount !== NaN) {
              amount += Math.max(line.original_amount - (line.receipt_amount || 0), 0);
            }
          });
          return [{
            path: input_path,
            value: String(amount),
            protect: 'N'
          }];
        }
      }
    ]
  };
})();
